@@ -37,7 +37,7 @@ rm -rf $DEST/generator/dist $DEST/generator/build $DEST/generator/nanopb_generat
 cp $DEST/generator-bin/nanopb_generator $DEST/generator-bin/protoc-gen-nanopb
 
 # Remove debugging symbols to reduce size of package
-( cd $DEST/generator-bin; strip *.so *.so.* )
+( cd $DEST/generator-bin; strip *.so* )
 
 # Tar it all up
 ( cd dist; tar -czf $VERSION.tar.gz $VERSION )
